'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _glMatrix = require('gl-matrix');

var _Buffer = require('./Buffer');

var _Buffer2 = _interopRequireDefault(_Buffer);

var _Box = require('./math/Box');

var _Box2 = _interopRequireDefault(_Box);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Используется для задания геометрий объектов.
 * В качестве данных используются {@link Buffer}.
 */

var Geometry = function () {
    function Geometry() {
        _classCallCheck(this, Geometry);

        /**
         * Словарь вида: название буфера - Buffer
         * @type {Object}
         */
        this.buffers = {};

        /**
         * Параллелепипед описывающий данную геометрию
         * @type {?Box}
         * @ignore
         */
        this._boundingBox = null;
    }

    /**
     * Сохраняет буфер в геометрию
     * @param {String} name Название буфера
     * @param {Buffer} buffer
     */


    _createClass(Geometry, [{
        key: 'setBuffer',
        value: function setBuffer(name, buffer) {
            this.buffers[name] = buffer;

            return this;
        }

        /**
         * Возвращает буфер из геометрии
         * @param {String} name Название буфера
         * @returns {Buffer}
         */

    }, {
        key: 'getBuffer',
        value: function getBuffer(name) {
            return this.buffers[name];
        }

        /**
         * Вычисляет буфер нормалей на основе буфера координат вершин (position)
         */

    }, {
        key: 'computeNormals',
        value: function computeNormals() {
            var positionBuffer = this.buffers.position;

            if (!positionBuffer) {
                return this;
            }

            var normals = new Float32Array(positionBuffer.length * positionBuffer.itemSize);

            var ab = _glMatrix.vec3.create();
            var cb = _glMatrix.vec3.create();
            var n = _glMatrix.vec3.create();

            for (var i = 0; i < positionBuffer.length; i += 3) {
                var triangle = positionBuffer.getTriangle(i / 3);

                _glMatrix.vec3.sub(ab, triangle[0], triangle[1]);
                _glMatrix.vec3.sub(cb, triangle[2], triangle[1]);
                _glMatrix.vec3.cross(n, ab, cb);
                _glMatrix.vec3.normalize(n, n);

                normals.set(n, i * 3);
                normals.set(n, (i + 1) * 3);
                normals.set(n, (i + 2) * 3);
            }

            this.setBuffer('normal', new _Buffer2.default(normals, 3));

            return this;
        }

        /**
         * Возвращает параллелепипед описывающий данную геометрию
         * @returns {Box}
         */

    }, {
        key: 'getBoundingBox',
        value: function getBoundingBox() {
            if (!this._boundingBox) {
                this.computeBoundingBox();
            }

            return this._boundingBox;
        }

        /**
         * Вычисляет параллелепипед описывающий данную геометрию на основе буфера координат вершин (position)
         * @returns {Box}
         */

    }, {
        key: 'computeBoundingBox',
        value: function computeBoundingBox() {
            var boundingBox = this._boundingBox = new _Box2.default();
            var positionBuffer = this.buffers.position;

            if (positionBuffer) {
                for (var i = 0; i < positionBuffer.length; i++) {
                    boundingBox.expandByPoint(positionBuffer.getElement(i));
                }
            }
        }

        /**
         * Соединяет данную геометрию с другой.
         * Осторожно, геометрии должны быть подобны, т.е. содержать одинаковые буферы.
         * @param {Geometry} geometry
         */

    }, {
        key: 'concat',
        value: function concat(geometry) {
            for (var type in this.buffers) {
                this.buffers[type].concat(geometry.buffers[type]);
            }
        }
    }]);

    return Geometry;
}();

exports.default = Geometry;
module.exports = exports['default'];