'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Object3D2 = require('./Object3D');

var _Object3D3 = _interopRequireDefault(_Object3D2);

var _glMatrix = require('gl-matrix');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Используется для отрисовки 3D объектов. Каждому мешу необходимо задать программу и геометрию.
 *
 * @extends {Object3D}
 */

var Mesh = function (_Object3D) {
    _inherits(Mesh, _Object3D);

    /**
     * @param {Geometry} geometry Геометрия меша
     * @param {Program} program Программа для отрисовки меша
     */

    function Mesh(geometry, program) {
        _classCallCheck(this, Mesh);

        /**
         * Геометрия меша
         * @type {Geometry}
         */

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Mesh).call(this));

        _this.geometry = geometry;

        /**
         * Программа для отрисовки меша
         * @type {Program}
         */
        _this.program = program;
        return _this;
    }

    /**
     * Вызывается рендером для подготовки и отрисовки объекта.
     * @param {State} state Текущие состояние рендера
     */


    _createClass(Mesh, [{
        key: 'render',
        value: function render(state) {
            var gl = state.gl;

            if (!this.visible) {
                return this;
            }

            if (this.worldMatrixNeedsUpdate) {
                this.updateWorldMatrix();
            }

            state.object = this;
            this.program.enable(state);

            gl.drawArrays(gl.TRIANGLES, 0, this.geometry.getBuffer('position').length);

            this.program.disable(gl);

            return this;
        }

        /**
         * Проверяет пересекает ли {@link Raycaster} данный объект, вносит все пересечения в массив intersects.
         * @param {Raycaster} raycaster
         * @param {Intersect[]} intersects
         * @param {Boolean} recursive Проверять ли пересечения с дочерними объектами
         */

    }, {
        key: 'raycast',
        value: function raycast(raycaster, intersects, recursive) {
            // get from https://github.com/mrdoob/three.js/blob/master/src/objects/Mesh.js

            var inverseMatrix = _glMatrix.mat4.create();
            _glMatrix.mat4.invert(inverseMatrix, this.worldMatrix);

            var ray = raycaster.ray.clone();
            ray.applyMatrix4(inverseMatrix);

            var boundingBox = this.geometry.getBoundingBox();

            if (!ray.intersectBox(boundingBox)) {
                return this;
            }

            var positionBuffer = this.geometry.buffers.position;

            for (var i = 0; i < positionBuffer.length; i += 3) {
                var triangle = positionBuffer.getTriangle(i / 3);

                var intersectionPoint = ray.intersectTriangle(triangle, false);

                if (!intersectionPoint) {
                    continue;
                }

                _glMatrix.vec3.transformMat4(intersectionPoint, intersectionPoint, this.worldMatrix);

                var distance = _glMatrix.vec3.dist(raycaster.ray.origin, intersectionPoint);

                if (distance < raycaster.precision || distance < raycaster.near || distance > raycaster.far) {
                    continue;
                }

                intersects.push({
                    distance: distance,
                    point: intersectionPoint,
                    object: this
                });
            }

            if (recursive) {
                this.children.forEach(function (child) {
                    return child.raycast(raycaster, intersects, recursive);
                });
            }

            return this;
        }

        /**
         * Вызывается на этапе рендеринга, чтобы определить к какому типу рендера принадлежит объект.
         * Меши разделяются на прозрачные и нет.
         *
         * @param {TypedObjects} typedObjects
         */

    }, {
        key: 'typifyForRender',
        value: function typifyForRender(typedObjects) {
            if (!this.visible) {
                return this;
            }

            this.program.typifyForRender(typedObjects, this);

            this.children.forEach(function (child) {
                return child.typifyForRender(typedObjects);
            });

            return this;
        }
    }]);

    return Mesh;
}(_Object3D3.default);

exports.default = Mesh;
module.exports = exports['default'];