'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Object3D2 = require('./Object3D');

var _Object3D3 = _interopRequireDefault(_Object3D2);

var _glMatrix = require('gl-matrix');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Используется для отрисовки спрайтов. Спрайты всегда рисуются лицевой стороной
 * и их размеры не зависят от положения. Т.е. координаты спрайта проецируются в плоскость экрана,
 * и уже на ней отрисовываются.
 *
 * @extends {Object3D}
 */

var Sprite = function (_Object3D) {
  _inherits(Sprite, _Object3D);

  /**
   * @param {SpriteProgram} program
   */

  function Sprite(program) {
    _classCallCheck(this, Sprite);

    /**
     * Программа отрисовки спрайта
     * @type {SpriteProgram}
     */

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Sprite).call(this));

    _this.program = program;

    /**
     * Смещение спрайта в плоскости экрана
     * @type {vec2}
     */
    _this.offset = _glMatrix.vec2.create();
    return _this;
  }

  _createClass(Sprite, [{
    key: 'render',
    value: function render(state) {
      // Если cпрайт невидим или у программы спрайта не установлена текстура, то не рендерим его
      if (!this.visible || !this.program.getTexture()) {
        return this;
      }

      if (this.worldMatrixNeedsUpdate) {
        this.updateWorldMatrix();
      }

      var gl = state.gl;

      state.object = this;

      this.program.enable(state);

      // draw for indices
      gl.drawElements(gl.TRIANGLES, 6, gl.UNSIGNED_SHORT, 0);

      this.program.disable(state.gl);

      return this;
    }

    /**
     * Вызывается на этапе рендеринга, чтобы определить к какому типу рендера принадлежит объект.
     * Спрайты рисуются отдельным рендером.
     *
     * @param {TypedObjects} typedObjects
     */

  }, {
    key: 'typifyForRender',
    value: function typifyForRender(typedObjects) {
      if (!this.visible) {
        return this;
      }

      this.program.typifyForRender(typedObjects, this);

      this.children.forEach(function (child) {
        return child.typifyForRender(typedObjects);
      });

      return this;
    }
  }]);

  return Sprite;
}(_Object3D3.default);

exports.default = Sprite;
module.exports = exports['default'];