'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _Object3D2 = require('../Object3D');

var _Object3D3 = _interopRequireDefault(_Object3D2);

var _glMatrix = require('gl-matrix');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Камера
 *
 * @extends Object3D
 */

var Camera = function (_Object3D) {
  _inherits(Camera, _Object3D);

  function Camera() {
    _classCallCheck(this, Camera);

    /**
     * Специфичный для камеры вектор, помогающий определить её положение
     * @type {vec3}
     */

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Camera).call(this));

    _this.up = _glMatrix.vec3.fromValues(0, 1, 0);

    /**
     * Проекционная матрица
     * @type {mat4}
     */
    _this.projectionMatrix = _glMatrix.mat4.create();

    /**
     * Матрица модель-вида использующаяся в шейдерах для получения конечного изображения
     * @type {mat4}
     */
    _this.modelViewMatrix = _glMatrix.mat4.create();

    /**
     * Матрица, обратная к worldMatrix
     * @type {mat4}
     */
    _this.worldInverseMatrix = _glMatrix.mat4.create();
    return _this;
  }

  /**
   * Обновляет проекционную матрицу. Обычно это нужно после изменения параметров камеры.
   * Используется в наследуемых классах.
   */


  _createClass(Camera, [{
    key: 'updateProjectionMatrix',
    value: function updateProjectionMatrix() {}

    /**
     * Обновляет глобальную матрицу объекта и матрицу модель-вида.
     * */

  }, {
    key: 'updateWorldMatrix',
    value: function updateWorldMatrix() {
      _get(Object.getPrototypeOf(Camera.prototype), 'updateWorldMatrix', this).call(this);

      _glMatrix.mat4.invert(this.worldInverseMatrix, this.worldMatrix);
      _glMatrix.mat4.multiply(this.modelViewMatrix, this.projectionMatrix, this.worldInverseMatrix);
    }

    /**
     * Проецирует вектор из глобальной системы координат на экран
     * @param {vec3} vector
     * @returns {vec3}
     */

  }, {
    key: 'project',
    value: function project(vector) {
      var result = _glMatrix.vec3.create();
      _glMatrix.vec3.transformMat4(result, vector, this.modelViewMatrix);
      return result;
    }

    /**
     * Проецирует вектор из системы координат экрана в глобальную
     * @param {vec3} vector
     * @returns {vec3}
     */

  }, {
    key: 'unproject',
    value: function unproject(vector) {
      var matrix = _glMatrix.mat4.create();
      var inverseMatrix = _glMatrix.mat4.create();
      var result = _glMatrix.vec3.create();

      _glMatrix.mat4.invert(inverseMatrix, this.projectionMatrix);
      _glMatrix.mat4.mul(matrix, this.worldMatrix, inverseMatrix);
      _glMatrix.vec3.transformMat4(result, vector, matrix);

      return result;
    }

    /**
     * Поворачивает камеру так, чтобы центр экрана точно смотрел на указанную позицию
     * @param {vec3} position
     */

  }, {
    key: 'lookAt',
    value: function lookAt(position) {
      var matrix4 = _glMatrix.mat4.create();
      var matrix3 = _glMatrix.mat3.create();
      _glMatrix.mat4.lookAt(matrix4, this.position, position, this.up);
      _glMatrix.mat4.transpose(matrix4, matrix4);
      _glMatrix.mat3.fromMat4(matrix3, matrix4);
      _glMatrix.quat.fromMat3(this.quaternion, matrix3);

      return this;
    }
  }]);

  return Camera;
}(_Object3D3.default);

exports.default = Camera;
module.exports = exports['default'];