'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _glMatrix = require('gl-matrix');

var _Math = require('./Math.js');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Класс для линий
 *
 * Взято из [three.js](https://github.com/mrdoob/three.js/blob/master/src/math/Line3.js)
 */

var Line3 = function () {
    /**
     * @param {vec3} start Начало
     * @param {vec3} end Конец
     */

    function Line3(start, end) {
        _classCallCheck(this, Line3);

        this.start = start || _glMatrix.vec3.create();
        this.end = end || _glMatrix.vec3.create();
    }

    /**
     * Возвращает параметр основанный на проекции ближайшей точки к линии.
     * Если clampToLine = true, возвращает параметр между 0 и 1.
     * @param {vec3} point
     * @param {Boolean} clampToLine
     * @returns {number}
     */


    _createClass(Line3, [{
        key: 'closestPointToPointParameter',
        value: function closestPointToPointParameter(point, clampToLine) {
            var startP = _glMatrix.vec3.create();
            var startEnd = _glMatrix.vec3.create();

            _glMatrix.vec3.sub(startP, point, this.start);
            _glMatrix.vec3.sub(startEnd, this.end, this.start);

            var startEnd2 = _glMatrix.vec3.dot(startEnd, startEnd);
            var startEndStartP = _glMatrix.vec3.dot(startEnd, startP);

            var t = startEndStartP / startEnd2;

            if (clampToLine) {
                t = (0, _Math.clamp)(t, 0, 1);
            }

            return t;
        }

        /**
         * Возвращает ближайшую точку на прямой заданную линией. Если clampToLine = true, возвращает точку
         * лежащую в пределах линии.
         * @param {vec3} point
         * @param {Boolean} clampToLine
         * @param {?vec3} optionalTarget Если указать параметр, то результат будет записан в него
         * @returns {vec3}
         */

    }, {
        key: 'closestPointToPoint',
        value: function closestPointToPoint(point, clampToLine, optionalTarget) {
            var t = this.closestPointToPointParameter(point, clampToLine);

            var result = optionalTarget || _glMatrix.vec3.create();
            result = this.delta(result);
            _glMatrix.vec3.scale(result, result, t);
            _glMatrix.vec3.add(result, result, this.start);

            return result;
        }

        /**
         * Вычитает вектор начала линии из конца
         * @param {?vec3} optionalTarget Если указать параметр, то результат будет записан в него
         * @returns {vec3}
         */

    }, {
        key: 'delta',
        value: function delta(optionalTarget) {
            var result = optionalTarget || _glMatrix.vec3.create();
            _glMatrix.vec3.sub(result, this.end, this.start);
            return result;
        }
    }]);

    return Line3;
}();

exports.default = Line3;
module.exports = exports['default'];