'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _glMatrix = require('gl-matrix');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Луч
 *
 * Взято из [three.js](https://github.com/mrdoob/three.js/blob/master/src/math/Ray.js)
 */

var Ray = function () {
    /**
     * @param {vec3} origin Позиция начала луча
     * @param {vec3} direction Направление луча
     */

    function Ray(origin, direction) {
        _classCallCheck(this, Ray);

        /**
         * Начало
         * @type {vec3}
         */
        this.origin = origin || _glMatrix.vec3.create();

        /**
         * Направление
         * @type {vec3}
         */
        this.direction = direction || _glMatrix.vec3.create();
    }

    /**
     * Возвращает копию луча
     * @returns {Ray}
     */


    _createClass(Ray, [{
        key: 'clone',
        value: function clone() {
            return new Ray(_glMatrix.vec3.clone(this.origin), _glMatrix.vec3.clone(this.direction));
        }

        /**
         * Ищёт точку на луче с заданным множителем
         * @param {Number} t Множитель
         * @returns {vec3}
         */

    }, {
        key: 'at',
        value: function at(t) {
            var result = _glMatrix.vec3.create();
            _glMatrix.vec3.scaleAndAdd(result, this.origin, this.direction, t);
            return result;
        }

        /**
         * Проверяет пересекает ли луч паралелепипед
         * @param {Box} box
         * @returns {?vec3} Точка пересечения или null
         */

    }, {
        key: 'intersectBox',
        value: function intersectBox(box) {
            var tmin = undefined,
                tmax = undefined,
                tymin = undefined,
                tymax = undefined,
                tzmin = undefined,
                tzmax = undefined;

            var invdirx = 1 / this.direction[0];
            var invdiry = 1 / this.direction[1];
            var invdirz = 1 / this.direction[2];

            var origin = this.origin;

            if (invdirx >= 0) {
                tmin = (box.min[0] - origin[0]) * invdirx;
                tmax = (box.max[0] - origin[0]) * invdirx;
            } else {
                tmin = (box.max[0] - origin[0]) * invdirx;
                tmax = (box.min[0] - origin[0]) * invdirx;
            }

            if (invdiry >= 0) {
                tymin = (box.min[1] - origin[1]) * invdiry;
                tymax = (box.max[1] - origin[1]) * invdiry;
            } else {
                tymin = (box.max[1] - origin[1]) * invdiry;
                tymax = (box.min[1] - origin[1]) * invdiry;
            }

            if (tmin > tymax || tymin > tmax) {
                return null;
            }

            // These lines also handle the case where tmin or tmax is NaN
            // (result of 0 * Infinity). x !== x returns true if x is NaN
            if (tymin > tmin || tmin !== tmin) {
                tmin = tymin;
            }

            if (tymax < tmax || tmax !== tmax) {
                tmax = tymax;
            }

            if (invdirz >= 0) {
                tzmin = (box.min[2] - origin[2]) * invdirz;
                tzmax = (box.max[2] - origin[2]) * invdirz;
            } else {
                tzmin = (box.max[2] - origin[2]) * invdirz;
                tzmax = (box.min[2] - origin[2]) * invdirz;
            }

            if (tmin > tzmax || tzmin > tmax) {
                return null;
            }

            if (tzmin > tmin || tmin !== tmin) {
                tmin = tzmin;
            }

            if (tzmax < tmax || tmax !== tmax) {
                tmax = tzmax;
            }

            // return point closest to the ray (positive side)
            if (tmax < 0) {
                return null;
            }

            return this.at(tmin >= 0 ? tmin : tmax);
        }

        /**
         * Изменяет направление луча с помощью матрицы
         * @param {mat4} matrix
         */

    }, {
        key: 'applyMatrix4',
        value: function applyMatrix4(matrix) {
            _glMatrix.vec3.add(this.direction, this.direction, this.origin);
            _glMatrix.vec3.transformMat4(this.direction, this.direction, matrix);
            _glMatrix.vec3.transformMat4(this.origin, this.origin, matrix);
            _glMatrix.vec3.sub(this.direction, this.direction, this.origin);
            _glMatrix.vec3.normalize(this.direction, this.direction);

            return this;
        }

        /**
         * Проверяет пересекает ли луч заданный треугольник
         * @param {vec3[]} triangle
         * @param {Boolean} [backfaceCulling=false] Если true, то луч может пересечь только переднюю сторону треугольника
         * @returns {?vec3} Точка пересечения или null
         */

    }, {
        key: 'intersectTriangle',
        value: function intersectTriangle(triangle, backfaceCulling) {
            // Compute the offset origin, edges, and normal.
            var edge1 = _glMatrix.vec3.create();
            var edge2 = _glMatrix.vec3.create();
            var normal = _glMatrix.vec3.create();

            _glMatrix.vec3.sub(edge1, triangle[1], triangle[0]);
            _glMatrix.vec3.sub(edge2, triangle[2], triangle[0]);
            _glMatrix.vec3.cross(normal, edge1, edge2);

            // Solve Q + t*D = b1*E1 + b2*E2 (Q = kDiff, D = ray direction,
            // E1 = kEdge1, E2 = kEdge2, N = Cross(E1,E2)) by
            //   |Dot(D,N)| * b1 = sign(Dot(D, N)) * Dot(D, Cross(Q, E2))
            //   |Dot(D,N)| * b2 = sign(Dot(D, N)) * Dot(D, Cross(E1, Q))
            //   |Dot(D,N)| * t = -sign(Dot(D, N)) * Dot(Q, N)
            var DdN = _glMatrix.vec3.dot(this.direction, normal);
            var sign = undefined;

            if (DdN > 0) {
                if (backfaceCulling) {
                    return null;
                }
                sign = 1;
            } else if (DdN < 0) {
                sign = -1;
                DdN = -DdN;
            } else {
                return null;
            }

            var diff = _glMatrix.vec3.create();
            _glMatrix.vec3.sub(diff, this.origin, triangle[0]);

            var cde2 = _glMatrix.vec3.create();
            _glMatrix.vec3.cross(cde2, diff, edge2);

            var DdQxE2 = sign * _glMatrix.vec3.dot(this.direction, cde2);

            // b1 < 0, no intersection
            if (DdQxE2 < 0) {
                return null;
            }

            var cde1 = _glMatrix.vec3.create();
            _glMatrix.vec3.cross(cde1, edge1, diff);
            var DdE1xQ = sign * _glMatrix.vec3.dot(this.direction, cde1);

            // b2 < 0, no intersection
            if (DdE1xQ < 0) {
                return null;
            }

            // b1+b2 > 1, no intersection
            if (DdQxE2 + DdE1xQ > DdN) {
                return null;
            }

            // Line intersects triangle, check if ray does.
            var QdN = -sign * _glMatrix.vec3.dot(diff, normal);

            // t < 0, no intersection
            if (QdN < 0) {
                return null;
            }

            // Ray intersects triangle.
            return this.at(QdN / DdN);
        }

        /**
         * Ищет расстояние от начала луча до плоскости
         * @param {Plane} plane
         * @returns {?Number}
         */

    }, {
        key: 'distanceToPlane',
        value: function distanceToPlane(plane) {
            var denominator = _glMatrix.vec3.dot(plane.normal, this.direction);

            if (denominator === 0) {
                // line is coplanar, return origin
                if (plane.distanceToPoint(this.origin) === 0) {
                    return 0;
                }

                // Null is preferable to undefined since undefined means.... it is undefined
                return null;
            }

            var t = -(_glMatrix.vec3.dot(this.origin, plane.normal) + plane.constant) / denominator;

            // Return if the ray never intersects the plane
            return t >= 0 ? t : null;
        }

        /**
         * Проверяет пересекает ли луч заданную плоскость
         * @param {Plane} plane
         * @returns {?vec3} Точка пересечения или null
         */

    }, {
        key: 'intersectPlane',
        value: function intersectPlane(plane) {
            var t = this.distanceToPlane(plane);

            if (t === null) {
                return null;
            }

            return this.at(t);
        }
    }]);

    return Ray;
}();

exports.default = Ray;
module.exports = exports['default'];