'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _shaders = require('../shaders');

var _glMatrix = require('gl-matrix');

var _Program2 = require('./Program');

var _Program3 = _interopRequireDefault(_Program2);

var _AmbientLight = require('../lights/AmbientLight');

var _AmbientLight2 = _interopRequireDefault(_AmbientLight);

var _DirectionalLight = require('../lights/DirectionalLight');

var _DirectionalLight2 = _interopRequireDefault(_DirectionalLight);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Более сложная программа для {@link Mesh}.
 *
 * {@link Geometry} меша использующего эту программу должна содержать следующие буферы:
 * 1. position - координаты вершин
 * 2. color - диффузная составляющая цвета в RGB для каждой из вершин, на неё влияет освещение
 * 3. emissive - фоновая составляющая цвета в RGB, на неё не влияет освещение
 *
 * Если программе задана текстура, то также должен быть доступны буферы:
 * 5. texture - 2х мерные координаты сопоставляющие координаты грани к координатам текстуры
 * 6. textureEnable - будет ли использоваться текстура для данной вершины,
 * принимает два значаения: 0 - нет, 1 - да
 *
 * @extends Program
 */

var ComplexMeshProgram = function (_Program) {
    _inherits(ComplexMeshProgram, _Program);

    function ComplexMeshProgram() {
        _classCallCheck(this, ComplexMeshProgram);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(ComplexMeshProgram).call(this));

        _this._attributeList = ['position', 'color', 'emissive'];
        _this._uniformList = ['uCamera', 'uPosition', 'uColorAlpha', 'uAmbientLightColor'];
        _this._shader = _shaders.complex;
        _this._texture = null;
        return _this;
    }

    /**
     * Задаёт текстуру программе
     * @param {Texture} texture
     */


    _createClass(ComplexMeshProgram, [{
        key: 'setTexture',
        value: function setTexture(texture) {
            this._texture = texture;

            return this;
        }

        /**
         * Возвращает текущую текстуру
         * @returns {?Texture}
         */

    }, {
        key: 'getTexture',
        value: function getTexture() {
            return this._texture;
        }
    }, {
        key: '_prepare',
        value: function _prepare(state) {
            this._enableLight(state);

            if (this._texture) {
                this._enableTexture();
            }

            _get(Object.getPrototypeOf(ComplexMeshProgram.prototype), '_prepare', this).call(this, state);
        }
    }, {
        key: '_enableLight',
        value: function _enableLight(_ref) {
            var scene = _ref.scene;

            var directionLightNumber = 0;

            scene.getLights().forEach(function (l) {
                if (l instanceof _DirectionalLight2.default) {
                    directionLightNumber++;
                }
            });

            this.define('directionLights', directionLightNumber);

            if (directionLightNumber > 0) {
                this._attributeList.push('normal');
                this._uniformList.push('uDirectionLightColors', 'uDirectionLightPositions', 'uNormalMatrix');
            }
        }
    }, {
        key: '_enableTexture',
        value: function _enableTexture() {
            this.define('texture');
            this._attributeList.push('texture', 'textureEnable');
            this._uniformList.push('uTexture');
        }
    }, {
        key: '_bindMesh',
        value: function _bindMesh(_ref2) {
            var _this2 = this;

            var gl = _ref2.gl;
            var scene = _ref2.scene;
            var camera = _ref2.camera;
            var object = _ref2.object;

            if (this._texture) {
                this._texture.enable(gl, this.uniforms.uTexture);
            }

            var lights = scene.getLights();

            if (lights.length) {
                (function () {
                    var directionLightsColor = [];
                    var directionLightsPosition = [];

                    lights.forEach(function (light) {
                        if (light instanceof _AmbientLight2.default) {
                            gl.uniform3fv(_this2.uniforms.uAmbientLightColor, light.color);
                        } else if (light instanceof _DirectionalLight2.default) {
                            directionLightsColor = directionLightsColor.concat(light.color);

                            var reverted = _glMatrix.vec3.create();
                            _glMatrix.vec3.scale(reverted, light.position, -1);
                            directionLightsPosition = directionLightsPosition.concat(Array.prototype.slice.call(reverted));
                        }
                    });

                    if (directionLightsColor.length && directionLightsPosition.length) {
                        var normalMatrix = _glMatrix.mat3.create();
                        _glMatrix.mat3.fromMat4(normalMatrix, object.worldMatrix);
                        _glMatrix.mat3.invert(normalMatrix, normalMatrix);
                        _glMatrix.mat3.transpose(normalMatrix, normalMatrix);
                        gl.uniformMatrix3fv(_this2.uniforms.uNormalMatrix, false, new Float32Array(normalMatrix));

                        object.geometry.getBuffer('normal').bind(gl, _this2.attributes.normal);
                    }

                    gl.uniform3fv(_this2.uniforms.uDirectionLightColors, new Float32Array(directionLightsColor));
                    gl.uniform3fv(_this2.uniforms.uDirectionLightPositions, new Float32Array(directionLightsPosition));
                })();
            }

            this._attributeList.forEach(function (name) {
                if (name !== 'normal') {
                    object.geometry.getBuffer(name).bind(gl, _this2.attributes[name]);
                }
            });

            gl.uniformMatrix4fv(this.uniforms.uPosition, false, new Float32Array(object.worldMatrix));
            gl.uniformMatrix4fv(this.uniforms.uCamera, false, new Float32Array(camera.modelViewMatrix));
            gl.uniform1f(this.uniforms.uColorAlpha, this.opacity);
        }
    }]);

    return ComplexMeshProgram;
}(_Program3.default);

exports.default = ComplexMeshProgram;
module.exports = exports['default'];