'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _definitions = require('./definitions');

var _definitions2 = _interopRequireDefault(_definitions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var cachedPrograms = {};

/**
 * Базовый класс для программ.
 * Программа инициализирует шейдеры, подготавливает и связывает данные с WebGL.
 */

var Program = function () {
    function Program() {
        _classCallCheck(this, Program);

        this._attributeList = [];
        this._uniformList = [];
        this._definitions = [];
        this._shader = null;

        /**
         * Прозрачность объекта отрисованного с помощью данной программы
         * @type {Number}
         */
        this.opacity = 1;
    }

    /**
     * Инициализирует программу. Связывает переменные шейдера с данными.
     * @param {State} state
     */


    _createClass(Program, [{
        key: 'enable',
        value: function enable(state) {
            var gl = state.gl;

            if (!this._shaderProgram) {
                this._prepare(state);
            }

            gl.useProgram(this._shaderProgram);

            for (var name in this.attributes) {
                gl.enableVertexAttribArray(this.attributes[name]);
            }

            this._bindMesh(state);

            return this;
        }

        /**
         * Отключает программу
         * @param {WebGLRenderingContext} gl
         */

    }, {
        key: 'disable',
        value: function disable(gl) {
            for (var name in this.attributes) {
                gl.disableVertexAttribArray(this.attributes[name]);
            }

            return this;
        }

        /**
         * Добавляет definitions в код шейдеров. Все добавления должны быть сделаны до первой инициализации.
         * @param {String} type
         * @param {Number | String} value
         */

    }, {
        key: 'define',
        value: function define(type, value) {
            if (_definitions2.default[type]) {
                this._definitions.push({ type: type, value: value });
            }

            return this;
        }

        /**
         * Вызывается объектом использующую данную программу,
         * чтобы определить к какому типу рендера принадлежит объект.
         * Самое простое разделение: на прозрачные и нет.
         *
         * @param {TypedObjects} typedObjects
         * @param {Object3D} object
         */

    }, {
        key: 'typifyForRender',
        value: function typifyForRender(typedObjects, object) {
            if (this.opacity === 1) {
                typedObjects.common.push(object);
            } else {
                typedObjects.transparent.push(object);
            }
        }
    }, {
        key: '_prepare',
        value: function _prepare(state) {
            this._prepareShaders(state);
            this._prepareAttributes(state);
            this._prepareUniforms(state);
        }
    }, {
        key: '_getCachedProgramKey',
        value: function _getCachedProgramKey() {
            return this.constructor.name + ':' + this._definitions.join(':');
        }
    }, {
        key: '_getCachedProgram',
        value: function _getCachedProgram() {
            return cachedPrograms[this._getCachedProgramKey()];
        }
    }, {
        key: '_prepareShaders',
        value: function _prepareShaders(_ref) {
            var gl = _ref.gl;

            var cachedProgram = this._getCachedProgram();

            if (cachedProgram && gl === cachedProgram.glContext) {
                this._shaderProgram = cachedProgram.program;
                return;
            }

            var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
            gl.shaderSource(fragmentShader, this._addDefinitions(this._shader.fragment));
            gl.compileShader(fragmentShader);

            if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {
                console.log(gl.getShaderInfoLog(fragmentShader));
            }

            var vertexShader = gl.createShader(gl.VERTEX_SHADER);
            gl.shaderSource(vertexShader, this._addDefinitions(this._shader.vertex));
            gl.compileShader(vertexShader);

            if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {
                console.log(gl.getShaderInfoLog(vertexShader));
            }

            this._shaderProgram = gl.createProgram();
            gl.attachShader(this._shaderProgram, vertexShader);
            gl.attachShader(this._shaderProgram, fragmentShader);
            gl.linkProgram(this._shaderProgram);

            if (!gl.getProgramParameter(this._shaderProgram, gl.LINK_STATUS)) {
                console.log('Could not initialize shaders');
            }

            cachedPrograms[this._getCachedProgramKey()] = {
                glContext: gl,
                program: this._shaderProgram
            };
        }
    }, {
        key: '_addDefinitions',
        value: function _addDefinitions(shader) {
            return this._definitions.map(function (def) {
                if (def.value !== undefined) {
                    return '#define ' + _definitions2.default[def.type] + ' ' + def.value;
                } else {
                    return '#define ' + _definitions2.default[def.type];
                }
            }).join('\n') + '\n' + shader;
        }
    }, {
        key: '_prepareAttributes',
        value: function _prepareAttributes(_ref2) {
            var _this = this;

            var gl = _ref2.gl;

            this.attributes = {};

            this._attributeList.forEach(function (name) {
                _this.attributes[name] = gl.getAttribLocation(_this._shaderProgram, name);
            });
        }
    }, {
        key: '_prepareUniforms',
        value: function _prepareUniforms(_ref3) {
            var _this2 = this;

            var gl = _ref3.gl;

            this.uniforms = {};

            this._uniformList.forEach(function (name) {
                _this2.uniforms[name] = gl.getUniformLocation(_this2._shaderProgram, name);
            });
        }
    }, {
        key: '_bindMesh',
        value: function _bindMesh(state) {
            this._bindAttributes(state);
            this._bindUniforms(state);
        }
    }, {
        key: '_bindAttributes',
        value: function _bindAttributes(_ref4) {
            var _this3 = this;

            var gl = _ref4.gl;
            var object = _ref4.object;

            this._attributeList.forEach(function (name) {
                object.geometry.getBuffer(name).bind(gl, _this3.attributes[name]);
            });
        }
    }, {
        key: '_bindUniforms',
        value: function _bindUniforms(_ref5) {
            var gl = _ref5.gl;
            var object = _ref5.object;
            var camera = _ref5.camera;

            gl.uniformMatrix4fv(this.uniforms.uPosition, false, new Float32Array(object.worldMatrix));
            gl.uniformMatrix4fv(this.uniforms.uCamera, false, new Float32Array(camera.modelViewMatrix));
            gl.uniform1f(this.uniforms.uColorAlpha, this.opacity);
        }
    }]);

    return Program;
}();

exports.default = Program;
module.exports = exports['default'];