'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _shaders = require('../shaders');

var _Geometry = require('../Geometry');

var _Geometry2 = _interopRequireDefault(_Geometry);

var _Buffer = require('../Buffer');

var _Buffer2 = _interopRequireDefault(_Buffer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Отдельный рендер, используется для отрисовки спрайтов.
 * @ignore
 */

var SpriteRenderer = function () {
    function SpriteRenderer() {
        _classCallCheck(this, SpriteRenderer);

        this._shader = _shaders.sprite;
        this._geometry = new _Geometry2.default();
        this._geometry.setBuffer('position', new _Buffer2.default(new Float32Array([-0.5, -0.5, 0, 0.5, -0.5, 0, 0.5, 0.5, 0, -0.5, 0.5, 0]), 3)).setBuffer('texture', new _Buffer2.default(new Float32Array([0, 0, 1, 0, 1, 1, 0, 1]), 2)).setBuffer('index', new _Buffer2.default(new Uint16Array([1, 2, 0, 3, 0, 2]), 1));

        this._geometry.getBuffer('index').type = _Buffer2.default.ElementArrayBuffer;
    }

    _createClass(SpriteRenderer, [{
        key: 'render',
        value: function render(state, renderObjects) {
            var gl = state.gl;
            var camera = state.camera;


            gl.disable(gl.DEPTH_TEST);

            gl.enable(gl.BLEND);
            gl.blendEquation(gl.FUNC_ADD);
            gl.blendFunc(gl.ONE, gl.ONE_MINUS_SRC_ALPHA);

            if (!this._shaderProgram) {
                this._prepare(state);
            }

            gl.useProgram(this._shaderProgram);

            gl.enableVertexAttribArray(this._attributes.position);
            gl.enableVertexAttribArray(this._attributes.texture);

            this._geometry.getBuffer('position').bind(gl, this._attributes.position);
            this._geometry.getBuffer('texture').bind(gl, this._attributes.texture);

            gl.uniformMatrix4fv(this._uniforms.uPCamera, false, new Float32Array(camera.modelViewMatrix));

            this._geometry.getBuffer('index').bind(gl);

            gl.activeTexture(gl.TEXTURE0);
            gl.uniform1i(this._uniforms.uTexture, 0);

            state.uniforms = this._uniforms;

            renderObjects.forEach(function (object) {
                return object.render(state);
            });

            gl.disableVertexAttribArray(this._attributes.position);
            gl.disableVertexAttribArray(this._attributes.texture);
        }
    }, {
        key: '_prepare',
        value: function _prepare(state) {
            this._prepareShaders(state);
            this._prepareAttributes(state);
            this._prepareUniforms(state);
        }
    }, {
        key: '_prepareShaders',
        value: function _prepareShaders(_ref) {
            var gl = _ref.gl;

            var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
            gl.shaderSource(fragmentShader, this._shader.fragment);
            gl.compileShader(fragmentShader);

            var vertexShader = gl.createShader(gl.VERTEX_SHADER);
            gl.shaderSource(vertexShader, this._shader.vertex);
            gl.compileShader(vertexShader);

            this._shaderProgram = gl.createProgram();
            gl.attachShader(this._shaderProgram, vertexShader);
            gl.attachShader(this._shaderProgram, fragmentShader);
            gl.linkProgram(this._shaderProgram);
        }
    }, {
        key: '_prepareAttributes',
        value: function _prepareAttributes(_ref2) {
            var gl = _ref2.gl;

            this._attributes = {
                position: gl.getAttribLocation(this._shaderProgram, 'position'),
                texture: gl.getAttribLocation(this._shaderProgram, 'texture')
            };
        }
    }, {
        key: '_prepareUniforms',
        value: function _prepareUniforms(_ref3) {
            var gl = _ref3.gl;

            this._uniforms = {
                uPCamera: gl.getUniformLocation(this._shaderProgram, 'uPCamera'),
                uPosition: gl.getUniformLocation(this._shaderProgram, 'uPosition'),
                uColorAlpha: gl.getUniformLocation(this._shaderProgram, 'uColorAlpha'),
                uTexture: gl.getUniformLocation(this._shaderProgram, 'uTexture'),
                uScale: gl.getUniformLocation(this._shaderProgram, 'uScale'),
                uHalfSize: gl.getUniformLocation(this._shaderProgram, 'uHalfSize'),
                uOffset: gl.getUniformLocation(this._shaderProgram, 'uOffset'),
                uSmoothing: gl.getUniformLocation(this._shaderProgram, 'uSmoothing')
            };
        }
    }]);

    return SpriteRenderer;
}();

exports.default = SpriteRenderer;
module.exports = exports['default'];